/*
 * Copyright (c) 2001-2005, Gaudenz Alder
 * Copyright (c) 2005, David Benson
 *
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.jgraph.example;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.net.URL;
import java.util.Hashtable;
import java.util.Map;
import java.util.Random;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.event.UndoableEditEvent;

import org.jgraph.JGraph;
import org.jgraph.example.GraphEd;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphUndoManager;
import org.jgraph.layout.JGraphLayoutAlgorithm;
import org.jgraph.layout.JGraphLayoutSettings;
import org.jgraph.layout.TreeLayoutAlgorithm;

public class LargeTreeLayoutExample extends GraphEd {

	// Actions which Change State
	protected Action tree;
	
	//Stores first unconnected cell available, root cell never is
	private int vertexCount = 1;
	//Stores first unconnected cell available
	private int edgeCount = 0;

	private final int numVertices = 10000;
	
	// Create array big enough for all cells
	private DefaultGraphCell[] cells = new DefaultGraphCell[numVertices*2];

	// Number of vertices on current level being worked on
	private int numVerticesLevel = 1;
	
	// Shared random object
	private Random random = new Random();
	
	private ConnectionSet cs = new ConnectionSet();
	private Map nested = new Hashtable();
	
	public static void main(String[] args) {
		// Construct Frame
		JFrame frame = new JFrame("Large Tree Layout Example");
		// Add an Editor Panel
		frame.getContentPane().add(new LargeTreeLayoutExample());
		// Set Default Size
		frame.setSize(520, 390);
		// Show Frame
		frame.setVisible(true);
	}

	public LargeTreeLayoutExample() {
		// Use Border Layout
		getContentPane().setLayout(new BorderLayout());

		// Construct the Graph
		// Construct Model and Graph
		MyModel model = new MyModel();
		graph = new MyGraph(model);
		// For performance, don't select inserted cells
		graph.getGraphLayoutCache().setSelectsAllInsertedCells(false);

		// Construct Command History
		//
		// Create a GraphUndoManager which also Updates the ToolBar
		undoManager = new GraphUndoManager() {
			// Override Superclass
			public void undoableEditHappened(UndoableEditEvent e) {
				// First Invoke Superclass
				super.undoableEditHappened(e);
				// Then Update Undo/Redo Buttons
				updateHistoryButtons();
			}
		};

		// Add Listeners to Graph
		//
		// Register UndoManager with the Model
		graph.getModel().addUndoableEditListener(undoManager);
		// Update ToolBar based on Selection Changes
		graph.getSelectionModel().addGraphSelectionListener(this);
		// Listen for Delete Keystroke when the Graph has Focus
		graph.addKeyListener(this);

		// Construct Panel
		//
		// Add a ToolBar
		getContentPane().add(createToolBar(), BorderLayout.NORTH);
		// Add the Graph as Center Component
		getContentPane().add(new JScrollPane(graph), BorderLayout.CENTER);
		insertInitialCells(graph);
	}
	
	public void insertInitialCells(JGraph graph) {
		// Control-drag should clone selection
		graph.setCloneable(true);

		// Enable edit without final RETURN keystroke
		graph.setInvokesStopCellEditing(true);

		// When over a cell, jump to its default port (we only have one, anyway)
		graph.setJumpToDefaultPort(true);

		for (int i=0; i < numVertices; i++) {
			DefaultGraphCell cell = new DefaultGraphCell(new Integer(i).toString());
			Map attributes = new Hashtable();
			Rectangle2D bounds;
			if (i != 0) {

				bounds = new Rectangle2D.Double(20+(128*(i%100)),
												20+(32*(i/100)),
												40,
												20);
			} else {
				bounds = new Rectangle2D.Double(20, 20, 40, 20);
			}
			GraphConstants.setBounds(attributes, bounds);
			GraphConstants.setBorderColor(attributes, Color.black);
			GraphConstants.setBackground(attributes, Color.yellow);
			nested.put(cell,attributes);
			// Add a Port
			DefaultPort port = new DefaultPort();
			cell.add(port);
			port.setParent(cell);
			cells[i] = cell;
		}

		connectNextLevel();
		
		// Insert the cells via the cache, so they get selected
		graph.getGraphLayoutCache().insert(cells,nested,cs,null,null);
	}
	
	
	protected void connectNextLevel() {
		// If we've connected all vertices stop connecting
		if (vertexCount < numVertices) {
			// Store the number of vertices on this level locally as the 
			// variable is going to be reused
			int localNumVerticesLevel = numVerticesLevel;
			numVerticesLevel = 0;
			int localVertexCount = vertexCount;
			// For each node in this level connect a random number of vertices
			for (	int i = localVertexCount-localNumVerticesLevel;
					i < localVertexCount;
					i ++) {
				connectChildrenVertices(cells[i]);
			}
			
			connectNextLevel();
		}
	}
	
	/**
	 * Connects the next <code>numChildren</code> free vertices as targets
	 * from the specified <code>parent</code>
	 * @param parent 
	 * @param numChildren
	 */
	protected void connectChildrenVertices(	DefaultGraphCell parent) {
		// If we've connected all vertices stop connecting
		if (vertexCount < numVertices) {
			// Make a list of child cells first. We don't recurse straight down
			// to leaves since we don't how deep the tree is. Instead, each
			// level is connected at a time. Increase the parameter in nextInt
			// makes the tree wider and shallower, decreasing makes it deeper
			// and narrower
			int numChildren = random.nextInt(2)+1;
			DefaultPort parentPort = (DefaultPort)(parent.getChildAt(0));
			for (int i=0; i < numChildren; i++) {
				// If we've connected all vertices stop connecting
				if (vertexCount < numVertices) {
					numVerticesLevel++;
					// Port of child i
					DefaultPort childPort = (DefaultPort)(cells[vertexCount++].getChildAt(0));
					
					DefaultEdge edge = new DefaultEdge();
					edge.setSource(parentPort);
					edge.setTarget(childPort);
					Map edgeAttrib = new Hashtable();
					nested.put(edge, edgeAttrib);
					int arrow = GraphConstants.ARROW_CLASSIC;
					GraphConstants.setLineEnd(edgeAttrib , arrow);
					GraphConstants.setEndFill(edgeAttrib, true);
					cs.connect(edge, parentPort, childPort);
					cells[(edgeCount++)+numVertices] = edge;
				}
			}
		}
	}
	
	// Ungroup the Groups in Cells and Select the Children
	public void treeLayout() {
		JGraphLayoutAlgorithm layout = new TreeLayoutAlgorithm();
		JGraphLayoutSettings settings = layout.createSettings();
		JFrame frame = (JFrame)SwingUtilities.getRoot(this);
		
		try {
			JDialog dialog = JGraphLayoutAlgorithm.createDialog(settings,
																frame,
																"Configure",
																"Close",
																"Apply");
			dialog.pack();
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			int x = (int)((double)(screen.width  - this.getWidth() ) / 2.0);
			int y = (int)((double)(screen.height - this.getHeight()) / 2.0);
			dialog.setLocation(x, y);
			
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		execute(layout);
	}

	//
	// ToolBar
	//
	public JToolBar createToolBar() {
		JToolBar toolbar = super.createToolBar();

		toolbar.addSeparator();

		// Apply tree layout
		URL treeUrl = getClass().getClassLoader().getResource(
				"org/jgraph/example/resources/tree.gif");
		ImageIcon treeIcon = new ImageIcon(treeUrl);
		tree = new AbstractAction("", treeIcon) {
			public void actionPerformed(ActionEvent e) {
				treeLayout();
			}
		};
		tree.setEnabled(true);
		toolbar.add(tree);

		return toolbar;
	}

	/** 
	 * Execute the algorithm
	 */
	protected void execute(JGraphLayoutAlgorithm layout) {
		JGraphLayoutAlgorithm.applyLayout(	graph,
											layout,
											cells,
											new Object[]{cells[0]} );
	}

 }
